# Copyright (C) 2018, 2019 Cumulus Networks, inc.
#
# All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
#
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

from cue.exceptions import NotFound


__all__ = [
    "noop_get", "noop_raw_get", "reusable_controller"
]


def reusable_controller(controller_function):
    """
    Mark a controller function as being reusable.

    This tells `spec_verify` not to complain when it sees the controller being
    used for multiple operations.
    """
    controller_function.is_reusable = True
    return controller_function


@reusable_controller
def noop_get(*args, **kwargs):
    raise NotFound


@reusable_controller
def noop_raw_get(*args, **kwargs):
    raise NotFound
