# Copyright (C) 2018, 2019 Cumulus Networks, inc.
#
# All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
#
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt


# Show the component's data structures in their native format.  The focus is on
# getting the data into Python.


import json

import logging
logger = logging.getLogger(__name__)


def copp_counters_get(ctx):
    result = {}
    output = ctx.sh.sudo("/usr/lib/cumulus/mlxcmd",
                         "--json", "traps", "show", "copp-stats")
    result = json.loads(str(output))

    return result


def copp_conf_get(ctx):
    copp_conf = {}
    st = {'TRUE': 'on', 'FALSE': 'off'}
    try:
        conf = ctx.fileops.load_file(
            '/etc/cumulus/control-plane/policers.conf')

        for line in conf.splitlines():
            if line.startswith('#') or line == '':
                continue
            k, v = line.partition(" = ")[::2]
            k_l = k.split('.')
            pol = k_l[1]
            attr = k_l[2]
            if attr == 'enable':
                copp_conf[pol] = {}
                copp_conf[pol][attr] = st[v]
            else:
                copp_conf[pol][attr] = v
        logger.debug(copp_conf)
        return copp_conf
    except FileNotFoundError:
        return {}


def trap_info_get(ctx):
    traps = {}
    st = {'TRUE': 'on', 'FALSE': 'off'}
    try:
        conf = ctx.fileops.load_file('/etc/cumulus/switchd.d/trap.conf')
        for line in conf.splitlines():
            if line.startswith('#') or line == '':
                continue
            k, v = line.partition(" = ")[::2]
            trap = k.split('.')[1]
            traps[trap] = {}
            traps[trap]['state'] = st[v]
        logger.debug(traps)
        return traps

    except FileNotFoundError:
        return {}
