#
# Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software product is a proprietary product of Nvidia Corporation
# and its affiliates (the "Company") and all right, title, and interest
# in and to the software product, including all associated intellectual
# property rights, are and shall remain exclusively with the Company.
#
# This software product is governed by the End User License Agreement
# provided with the software product.
# Copyright (C) 2021 Cumulus Networks, inc.
# All Rights reserved.
from pydash import py_


###############################
# Control-plane
###############################
def control_plane_get(ctx, rev):
    if rev == 'operational':
        return ctx.copp_trap_v1.getCpInfo()
    return ctx.config_v1.getControlPlane(rev)


def trap_get(ctx, rev):
    control_plane = control_plane_get(ctx, rev)
    return py_.get(control_plane, "trap", {})


def trap_id_get(ctx, rev, trap_id):
    trap = trap_get(ctx, rev)
    try:
        return trap[trap_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


def policer_get(ctx, rev):
    control_plane = control_plane_get(ctx, rev)
    return py_.get(control_plane, "policer", {})


def policer_id_get(ctx, rev, policer_id):
    policer = policer_get(ctx, rev)
    try:
        return policer[policer_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


def policer_id_statistics_get(ctx, rev, policer_id):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)

    statistics = {}
    try:
        policer = policer_id_get(ctx, rev, policer_id)
        statistics = py_.get(policer, "statistics", {})
    except ctx.errors.ItemDoesNotExist:
        statistics = {}
    return statistics


###############################
# Acl
###############################


def acls_get(ctx, rev):
    if rev == 'operational':
        # Add the operational state when FUnit is ready
        rev = 'applied'
    return ctx.config_v1.getAcls(rev)


def acl_get(ctx, rev, acl_id):
    if rev != "operational":
        return ctx.config_v1.getAcl(rev, acl_id)

    # Add the operational state when FUnit is ready
    return {}


####################################
# ACL rules
####################################

def acl_rules_get(ctx, rev, acl_id):
    if rev == 'operational':
        # Add the operational state when FUnit is ready
        rev = 'applied'

    acl = acl_get(ctx, rev, acl_id)
    return acl["rule"]


####################################
# ACL rule
####################################

def acl_rule_get(ctx, rev, acl_id, rule_id):
    rules = acl_rules_get(ctx, rev, acl_id)
    try:
        return rules[rule_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


####################################
# ACL rule match
####################################

def acl_match_get(ctx, rev, acl_id, rule_id):
    if rev != "operational":
        rule = acl_rule_get(ctx, rev, acl_id, rule_id)
        return rule["match"]

    return {}


####################################
# ACL rule match ip
####################################

def acl_match_ip_get(ctx, rev, acl_id, rule_id):
    if rev != "operational":
        match = acl_match_get(ctx, rev, acl_id, rule_id)
        return match["ip"]

    return {}


####################################
# ACL rule match ip source ports
####################################

def acl_match_ip_sports_get(ctx, rev, acl_id, rule_id):
    if rev == 'operational':
        # Add the operational state when FUnit is ready
        rev = 'applied'
    ip_match = acl_match_ip_get(ctx, rev, acl_id, rule_id)
    return ip_match["source-port"]


####################################
# ACL rule match ip source port
####################################

def acl_match_ip_sport_get(ctx, rev, acl_id, rule_id, ip_port_id):
    if rev != "operational":
        sports = acl_match_ip_sports_get(ctx, rev, acl_id, rule_id)
        try:
            return sports[ip_port_id]
        except KeyError:
            raise ctx.errors.ItemDoesNotExist

    return {}


####################################
# ACL rule match ip dest ports
####################################

def acl_match_ip_dports_get(ctx, rev, acl_id, rule_id):
    if rev == 'operational':
        # Add the operational state when FUnit is ready
        rev = 'applied'
    ip_match = acl_match_ip_get(ctx, rev, acl_id, rule_id)
    return ip_match["dest-port"]


#####################################
# ACL rule match ip dest port
####################################

def acl_match_ip_dport_get(ctx, rev, acl_id, rule_id, ip_port_id):
    if rev != "operational":
        dports = acl_match_ip_dports_get(ctx, rev, acl_id, rule_id)
        try:
            return dports[ip_port_id]
        except KeyError:
            raise ctx.errors.ItemDoesNotExist

    return {}


####################################
# ACL rule match ip fragment
####################################

def acl_match_ip_fragment_get(ctx, rev, acl_id, rule_id):
    if rev != "operational":
        ip_match = acl_match_ip_get(ctx, rev, acl_id, rule_id)
        return ip_match["fragment"]

    return {}


####################################
# ACL rule match ip ecn
####################################

def acl_match_ip_ecn_get(ctx, rev, acl_id, rule_id):
    if rev != "operational":
        ip_match = acl_match_ip_get(ctx, rev, acl_id, rule_id)
        return ip_match["ecn"]

    return {}


def acl_match_ip_ecn_flags_get(ctx, rev, acl_id, rule_id):
    if rev != "operational":
        ecn_match = acl_match_ip_ecn_get(ctx, rev, acl_id, rule_id)
        return ecn_match["flags"]

    return {}


####################################
# ACL rule match ip tcp
####################################

def acl_match_ip_tcp_get(ctx, rev, acl_id, rule_id):
    if rev != "operational":
        ip_match = acl_match_ip_get(ctx, rev, acl_id, rule_id)
        return ip_match["tcp"]

    return {}


####################################
# ACL rule match ip tcp flags
####################################

def acl_match_ip_tcp_flags_get(ctx, rev, acl_id, rule_id):
    if rev != "operational":
        tcp_match = acl_match_ip_tcp_get(ctx, rev, acl_id, rule_id)
        return tcp_match["flags"]

    return {}


####################################
# ACL rule match ip tcp flags mask
####################################

def acl_match_ip_tcp_mask_get(ctx, rev, acl_id, rule_id):
    if rev != "operational":
        tcp_match = acl_match_ip_tcp_get(ctx, rev, acl_id, rule_id)
        return tcp_match["mask"]

    return {}


###################################
# ACL rule match mac
####################################

def acl_match_mac_get(ctx, rev, acl_id, rule_id):
    if rev != "operational":
        match = acl_match_get(ctx, rev, acl_id, rule_id)
        return match["mac"]

    return {}


####################################
# ACL rule action
####################################

def acl_action_get(ctx, rev, acl_id, rule_id):
    if rev != "operational":
        rule = acl_rule_get(ctx, rev, acl_id, rule_id)
        return rule["action"]

    return {}


####################################
# ACL rule action permit
####################################

def acl_action_permit_get(ctx, rev, acl_id, rule_id):
    if rev != "operational":
        action = acl_action_get(ctx, rev, acl_id, rule_id)
        return action["permit"]

    return {}


####################################
# ACL rule action deny
####################################

def acl_action_deny_get(ctx, rev, acl_id, rule_id):
    if rev != "operational":
        action = acl_action_get(ctx, rev, acl_id, rule_id)
        return action["deny"]

    return {}


####################################
# ACL rule action log
####################################

def acl_action_log_get(ctx, rev, acl_id, rule_id):
    if rev != "operational":
        action = acl_action_get(ctx, rev, acl_id, rule_id)
        return action["log"]

    return {}


####################################
# ACL rule action set
####################################

def acl_action_set_get(ctx, rev, acl_id, rule_id):
    if rev != "operational":
        action = acl_action_get(ctx, rev, acl_id, rule_id)
        return action["set"]

    return {}


####################################
# ACL rule action erspan
####################################

def acl_action_erspan_get(ctx, rev, acl_id, rule_id):
    if rev != "operational":
        action = acl_action_get(ctx, rev, acl_id, rule_id)
        return action["erspan"]

    return {}


####################################
# ACL rule action police
####################################

def acl_action_police_get(ctx, rev, acl_id, rule_id):
    if rev != "operational":
        action = acl_action_get(ctx, rev, acl_id, rule_id)
        return action["police"]

    return {}
