#
# Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software product is a proprietary product of Nvidia Corporation
# and its affiliates (the "Company") and all right, title, and interest
# in and to the software product, including all associated intellectual
# property rights, are and shall remain exclusively with the Company.
#
# This software product is governed by the End User License Agreement
# provided with the software product.
# Copyright (C) 2021 Cumulus Networks, inc.
# All Rights reserved.


###############################
# DHCP Relays
###############################

def dhcp_relays_get(ctx, rev):
    if rev != "operational":
        return ctx.config_v1.getDhcpRelays(rev)

    return ctx.dhcp_relay_v1.getRelays()


def dhcp_relay_get(ctx, rev, vrf_id):
    if rev != "operational":
        return ctx.config_v1.getDhcpRelay(rev, vrf_id)

    return ctx.dhcp_relay_v1.getRelay(vrf_id)


####################################
# DHCP Relay/server
####################################

def servers_get(ctx, rev, vrf_id):
    if rev != "operational":
        relay = ctx.config_v1.getDhcpRelay(rev, vrf_id)
        return relay["server"]

    relay = ctx.dhcp_relay_v1.getRelay(vrf_id)
    return relay["server"]


def server_get(ctx, rev, vrf_id, server_id):
    servers = servers_get(ctx, rev, vrf_id)
    try:
        return servers[server_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


####################################
# DHCP Relay/interface
####################################

def interfaces_get(ctx, rev, vrf_id):
    if rev != "operational":
        relay = ctx.config_v1.getDhcpRelay(rev, vrf_id)
        return relay["interface"]

    relay = ctx.dhcp_relay_v1.getRelay(vrf_id)
    return relay["interface"]


def interface_get(ctx, rev, vrf_id, interface_id):
    interfaces = interfaces_get(ctx, rev, vrf_id)
    try:
        return interfaces[interface_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


####################################
# DHCP Relay/giaddress-interface
####################################

def giaddrintfs_get(ctx, rev, vrf_id):
    if rev != "operational":
        relay = ctx.config_v1.getDhcpRelay(rev, vrf_id)
        return relay["giaddress-interface"]

    relay = ctx.dhcp_relay_v1.getRelay(vrf_id)
    return relay["giaddress-interface"]


def giaddrintf_get(ctx, rev, vrf_id, interface_id):
    gis = giaddrintfs_get(ctx, rev, vrf_id)
    try:
        return gis[interface_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist
