#
# Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software product is a proprietary product of Nvidia Corporation
# and its affiliates (the "Company") and all right, title, and interest
# in and to the software product, including all associated intellectual
# property rights, are and shall remain exclusively with the Company.
#
# This software product is governed by the End User License Agreement
# provided with the software product.
# Copyright (C) 2021 Cumulus Networks, inc.
# All Rights reserved.


###############################
# DHCP Relays
###############################

def dhcp_relays_get(ctx, rev):
    if rev != "operational":
        return ctx.config_v1.getDhcpRelays6(rev)

    return ctx.dhcp_relay_v1.getRelays6()


def dhcp_relay_get(ctx, rev, vrf_id):
    if rev != "operational":
        return ctx.config_v1.getDhcpRelay6(rev, vrf_id)

    return ctx.dhcp_relay_v1.getRelay6(vrf_id)


####################################
# DHCP Relay/interface
####################################

def interfaces_get(ctx, rev, vrf_id):
    if rev != "operational":
        relay = ctx.config_v1.getDhcpRelay6(rev, vrf_id)
        return relay["interface"]

    relay = ctx.dhcp_relay_v1.getRelay6(vrf_id)
    return relay["interface"]


####################################
# DHCP Relay/interface/upstream
####################################

def upstream_get(ctx, rev, vrf_id):
    interfaces = interfaces_get(ctx, rev, vrf_id)
    return interfaces["upstream"]


##################################################
# DHCP Relay/interface/upstream/{interface-id}
##################################################

def upst_interface_get(ctx, rev, vrf_id, interface_id):
    upst = upstream_get(ctx, rev, vrf_id)
    try:
        return upst[interface_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


####################################
# DHCP Relay/interface/downstream
####################################

def downstream_get(ctx, rev, vrf_id):
    interfaces = interfaces_get(ctx, rev, vrf_id)
    return interfaces["downstream"]


##################################################
# DHCP Relay/interface/downstream/{interface-id}
##################################################

def dnst_interface_get(ctx, rev, vrf_id, interface_id):
    dnst = downstream_get(ctx, rev, vrf_id)
    try:
        return dnst[interface_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist
