# Copyright (C) 2018-2020 Cumulus Networks, inc.
#
# All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
#
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt


###############################
# DHCP6 Servers
###############################

def servers_get(ctx, rev):
    if rev != "operational":
        return ctx.config_v1.getDhcpServers6(rev)

    return ctx.isc_dhcp_v1.getServers6()


###############################
# DHCP6 Server/{vrf-id}
###############################

def server_get(ctx, rev, vrf_id):
    if rev != "operational":
        return ctx.config_v1.getDhcpServer6(rev, vrf_id)

    return ctx.isc_dhcp_v1.getServer6(vrf_id)


###############################
# Server/{vrf-id}/interfaces/
###############################

def interfaces_get(ctx, rev, vrf_id):
    if rev != "operational":
        s = ctx.config_v1.getDhcpServer6(rev, vrf_id)
        return s["interface"]

    s = ctx.isc_dhcp_v1.getServer6(vrf_id)
    return s["interface"]


####################################################
# Server/{vrf-id}/interfaces/{dhcp-interface-id}
####################################################

def interface_get(ctx, rev, vrf_id, interface_id):
    s = interfaces_get(ctx, rev, vrf_id)
    try:
        return s[interface_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


###############################
# DHCP6 Server/{vrf-id}/pool
###############################

def pools_get(ctx, rev, vrf_id):
    if rev != "operational":
        s = ctx.config_v1.getDhcpServer6(rev, vrf_id)
        return s["pool"]

    s = ctx.isc_dhcp_v1.getServer6(vrf_id)
    return s["pool"]


###############################
# DHCP6 Server/{vrf-id}/pool
###############################

def pool_get(ctx, rev, vrf_id, pool_id):
    pools = pools_get(ctx, rev, vrf_id)
    try:
        return pools[pool_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


###############################
# Pool - Domain name servers
###############################

def pool_domain_name_servers_get(ctx, rev, vrf_id, pool_id):
    pool = pool_get(ctx, rev, vrf_id, pool_id)
    return pool['domain-name-server']


###############################
# Pool - Domain name server
###############################

def pool_domain_name_server_get(ctx, rev, vrf_id, pool_id,
                                server_id):
    dn_servers = pool_domain_name_servers_get(ctx, rev, vrf_id, pool_id)
    try:
        return dn_servers[server_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


###############################
# Pool - Domain names
###############################

def pool_domain_names_get(ctx, rev, vrf_id, pool_id):
    pool = pool_get(ctx, rev, vrf_id, pool_id)
    return pool['domain-name']


###############################
# Pool - Domain name
###############################

def pool_domain_name_get(ctx, rev, vrf_id, pool_id, domain_name_id):
    domain_names = pool_domain_names_get(ctx, rev, vrf_id, pool_id)
    try:
        return domain_names[domain_name_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


###############################
# DHCP6 Pool - Ranges
###############################

def ranges_get(ctx, rev, vrf_id, pool_id):
    pool = pool_get(ctx, rev, vrf_id, pool_id)
    return pool["range"]


###############################
# DHCP6 Pool - Range
###############################

def range_get(ctx, rev, vrf_id, pool_id, range_id):
    ranges = ranges_get(ctx, rev, vrf_id, pool_id)
    try:
        return ranges[range_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


###############################
# DHCP6 Pool/domain-name
###############################

def domain_names_get(ctx, rev, vrf_id):
    if rev != "operational":
        s = ctx.config_v1.getDhcpServer6(rev, vrf_id)
        return s["domain-name"]

    s = ctx.isc_dhcp_v1.getServer6(vrf_id)
    return s["domain-name"]


######################################
# DHCP6 Pool/domain-name/
######################################

def domain_name_get(ctx, rev, vrf_id, domain_name_id):
    domain_names = domain_names_get(ctx, rev, vrf_id)
    try:
        return domain_names[domain_name_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


######################################
# DHCP6 {vrf-id}/Pool/domain-name-server/
######################################

def domain_name_servers_get(ctx, rev, vrf_id):
    if rev != "operational":
        s = ctx.config_v1.getDhcpServer6(rev, vrf_id)
        return s["domain-name-server"]

    s = ctx.isc_dhcp_v1.getServer6(vrf_id)
    return s["domain-name-server"]


def domain_name_server_get(ctx, rev, vrf_id, server_id):
    domain_name_servers = domain_name_servers_get(ctx, rev, vrf_id)
    try:
        return domain_name_servers[server_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


def statics_get(ctx, rev, vrf_id):
    if rev != "operational":
        s = ctx.config_v1.getDhcpServer6(rev, vrf_id)
        return s["static"]

    s = ctx.isc_dhcp_v1.getServer6(vrf_id)
    return s["static"]


def static_get(ctx, rev, vrf_id, static_id):
    statics = statics_get(ctx, rev, vrf_id)
    try:
        return statics[static_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist
