# Copyright (C) 2018-2021 Cumulus Networks, inc
#
# All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
#
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt


from cue.exceptions import NotFound


###############################
# Evpn
###############################
def evpn_get(ctx, rev):
    if rev == 'operational':
        # EVPN config does not have a operational revision
        rev = 'applied'
    return ctx.config_v1.getEvpn(rev)


###############################
# Evpn Route Advertise
###############################
def evpn_ra_get(ctx, rev):
    return evpn_get(ctx, rev)["route-advertise"]


###############################
# Evpn DAD
###############################
def evpn_dad_get(ctx, rev):
    output = evpn_get(ctx, rev)
    try:
        return output["dad"]
    except KeyError:
        raise NotFound


###############################
# Evpn DAD Duplicate Action
###############################
def evpn_dad_da_get(ctx, rev):
    output = evpn_dad_get(ctx, rev)
    try:
        return output["duplicate-action"]
    except KeyError:
        raise NotFound


###############################
# Evpn DAD Duplicate Action Freeze
###############################
def evpn_dad_da_freeze_get(ctx, rev):
    output = evpn_dad_da_get(ctx, rev)
    try:
        return output["freeze"]
    except KeyError:
        raise NotFound


###############################
# Evpn EVIs
###############################
def evpn_evis_get(ctx, rev):
    output = evpn_get(ctx, rev)
    try:
        return output["evi"]
    except KeyError:
        raise NotFound


###############################
# Evpn EVI
###############################
def evpn_evi_get(ctx, rev, evi_id):
    evis = evpn_evis_get(ctx, rev)
    try:
        return evis[evi_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


###############################
# Evpn EVI Route advertise
###############################
def evpn_evi_ra_get(ctx, rev, evi_id):
    evi = evpn_evi_get(ctx, rev, evi_id)
    try:
        return evi["route-advertise"]
    except KeyError:
        raise NotFound


###############################
# Evpn EVI Route target
###############################
def evpn_evi_rt_get(ctx, rev, evi_id):
    evi = evpn_evi_get(ctx, rev, evi_id)
    try:
        return evi["route-target"]
    except KeyError:
        raise NotFound


###############################
# Evpn EVI Route target Export
###############################
def evpn_evi_rt_exports_get(ctx, rev, evi_id):
    rt = evpn_evi_rt_get(ctx, rev, evi_id)
    try:
        return rt["export"]
    except KeyError:
        raise NotFound


###############################
# Evpn EVI Route target Export
###############################
def evpn_evi_rt_export_get(ctx, rev, evi_id, rt_id):
    rt = evpn_evi_rt_exports_get(ctx, rev, evi_id)
    try:
        return rt[rt_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


###############################
# Evpn EVI Route target Export
###############################
def evpn_evi_rt_imports_get(ctx, rev, evi_id):
    rt = evpn_evi_rt_get(ctx, rev, evi_id)
    try:
        return rt["import"]
    except KeyError:
        raise NotFound


###############################
# Evpn EVI Route target Export
###############################
def evpn_evi_rt_import_get(ctx, rev, evi_id, rt_id):
    rt = evpn_evi_rt_imports_get(ctx, rev, evi_id)
    try:
        return rt[rt_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


###############################
# Evpn EVI Route target Both
###############################
def evpn_evi_rt_boths_get(ctx, rev, evi_id):
    rt = evpn_evi_rt_get(ctx, rev, evi_id)
    try:
        return rt["both"]
    except KeyError:
        raise NotFound


###############################
# Evpn EVI Route target both
###############################
def evpn_evi_rt_both_get(ctx, rev, evi_id, rt_id):
    rt = evpn_evi_rt_boths_get(ctx, rev, evi_id)
    try:
        return rt[rt_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


###############################
# Evpn Multihoming
###############################
def evpn_mh_get(ctx, rev):
    return evpn_get(ctx, rev)["multihoming"]


###############################
# Evpn Multihoming ead-evi-route
###############################
def evpn_mh_ead_get(ctx, rev):
    return evpn_mh_get(ctx, rev)["ead-evi-route"]


###############################
# Evpn Multihoming segment
###############################
def evpn_mh_segment_get(ctx, rev):
    return evpn_mh_get(ctx, rev)["segment"]
