# Copyright (C) 2018-2021 Cumulus Networks, inc
#
# All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
#
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt


###############################
# MLAG Verbose
###############################
def mlag_get_verbose(ctx, rev, option):
    return ctx.netlink_v1.getMlag(option)


###############################
# MLAG
###############################
def mlag_get(ctx, rev):
    if rev != "operational":
        return ctx.config_v1.getMlag(rev)
    return mlag_get_verbose(ctx, rev, "status")


###############################
# Backups
###############################
def backups_get(ctx, rev):
    if rev != "operational":
        return ctx.config_v1.getMlag(rev, "backup")
    return mlag_get_verbose(ctx, rev, "backup")["backup"]


###############################
# Backup
###############################
def backup_get(ctx, rev, backup_id):
    backups = backups_get(ctx, rev)
    try:
        return backups[backup_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


###############################
# MLAG Macs
###############################
def mlag_mac_db_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)

    return mlag_get_verbose(ctx, rev, "fdb")["fdb"]


###############################
# MLAG Local Macs
###############################
def mlag_localmacs_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)

    return mlag_mac_db_get(ctx, rev)['local']


###############################
# MLAG Peer Macs
###############################
def mlag_peermacs_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)

    return mlag_mac_db_get(ctx, rev)['peer']


###############################
# MLAG Permanent Macs
###############################
def mlag_permanentmacs_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)

    return mlag_mac_db_get(ctx, rev)["permanent"]


###############################
# MLAG Mdb
###############################
def mlag_mdb_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)

    return mlag_get_verbose(ctx, rev, "mdb")["mdb"]


###############################
# MLAG Local Mcast Groups
###############################
def mlag_localmcastgroups_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)

    return mlag_mdb_get(ctx, rev)["local"]


###############################
# MLAG Peer Mcast Groups
###############################
def mlag_peermcastgroups_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)

    return mlag_mdb_get(ctx, rev)["peer"]


###############################
# MLAG Vni Db
###############################
def mlag_vnidb_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)

    return mlag_get_verbose(ctx, rev, "vni")["vni"]


###############################
# MLAG Our Vnis
###############################
def mlag_localvnis_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)

    return mlag_vnidb_get(ctx, rev)["local"]


###############################
# MLAG Peer Vnis
###############################
def mlag_peervnis_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)

    return mlag_vnidb_get(ctx, rev)["peer"]


###############################
# MLAG Lacp DB
###############################
def mlag_lacpdb_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)

    return mlag_get_verbose(ctx, rev, "lacpdb")["lacpdb"]


###############################
# MLAG Our Lacp DB
###############################
def mlag_locallacpinfo_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)

    return mlag_lacpdb_get(ctx, rev)['local']


###############################
# MLAG Peer Lacp DB
###############################
def mlag_peerlacpinfo_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)

    return mlag_lacpdb_get(ctx, rev)['peer']


###############################
# MLAG Multicast Router Ports
###############################
def mlag_routerportdb_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)
    option = "multicast-router-port"
    return mlag_get_verbose(ctx, rev, option)[option]


###############################
# MLAG Our Router Ports
###############################
def mlag_localrouterports_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)

    return mlag_routerportdb_get(ctx, rev)['local']


###############################
# MLAG Peer Router Ports
###############################
def mlag_peerrouterports_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)

    return mlag_routerportdb_get(ctx, rev)['peer']


###############################
# MLAG Neighs
###############################
def mlag_neighdb_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)

    return mlag_get_verbose(ctx, rev, "neighbor")["neighbor"]


###############################
# MLAG Permanent Neighs
###############################
def mlag_dynamicneighs_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)

    return mlag_neighdb_get(ctx, rev)['dynamic']


###############################
# MLAG Permanent Neighs
###############################
def mlag_permanentneighs_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)

    return mlag_neighdb_get(ctx, rev)['permanent']
