# Copyright (C) 2018-2020 Cumulus Networks, inc.
#
# All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
#
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt


###############################
# NTP
###############################

def ntps_get(ctx, rev):
    if rev != "operational":
        return ctx.config_v1.getNtps(rev)

    return ctx.ntp_v1.getNtpConfigurations()


###############################
# NTP/{vrf-id}
###############################

def ntp_get(ctx, rev, vrf_id):
    if rev != "operational":
        return ctx.config_v1.getNtp(rev, vrf_id)

    return ctx.ntp_v1.getNtpConfiguration(vrf_id)


###############################
# NTP Servers
###############################

def servers_get(ctx, rev, vrf_id):
    if rev != "operational":
        ntp = ctx.config_v1.getNtp(rev, vrf_id)
        return ntp["server"]

    return ctx.ntp_v1.getNtpServers()


###############################
# NTP Server
###############################

def server_get(ctx, rev, vrf_id, server_id):
    servers = servers_get(ctx, rev, vrf_id)
    try:
        return servers[server_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


###############################
# NTP Pools
###############################

def pools_get(ctx, rev, vrf_id):
    if rev != "operational":
        return ctx.config_v1.getNtp(rev, vrf_id)["pool"]

    return ctx.ntp_v1.getNtpConfiguration(vrf_id)["pool"]


###############################
# NTP Pool
###############################

def pool_get(ctx, rev, vrf_id, server_id):
    pools = pools_get(ctx, rev, vrf_id)
    try:
        return pools[server_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist
