# Copyright (C) 2018, 2019 Cumulus Networks, inc.
#
# All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
#
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt


from cue import utils


def platform_get(ctx, rev):
    if rev != "operational":
        return ctx.config_v1.getPlatform(rev)

    return ctx.env_v1.getPlatform()


def software_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)
    return ctx.env_v1.getSoftware()


def installed_packages_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)
    return ctx.env_v1.getInstalledPackages()


def installed_package_get(ctx, rev, installed_id):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)
    return ctx.env_v1.getInstalledPackage(installed_id)


###############################
# Capabilities
###############################

def capabilities_get(ctx, rev):
    ops = ctx.cue_v1._ops
    ops.pre_get_oper_only(rev)
    return utils.fast_merge(
        ctx.platform_v1.getIfupdown2Capabilities()
    )


###############################
# environment
###############################

def environment_get(ctx, rev):
    return {
        "fan": ctx.cue_v1.getFans(rev),
        "sensor": ctx.cue_v1.getSensors(rev),
        "psu": ctx.cue_v1.getPsus(rev),
        "led": ctx.cue_v1.getLeds(rev),
    }
