#
# Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software product is a proprietary product of Nvidia Corporation
# and its affiliates (the "Company") and all right, title, and interest
# in and to the software product, including all associated intellectual
# property rights, are and shall remain exclusively with the Company.
#
# This software product is governed by the End User License Agreement
# provided with the software product.
# Copyright (C) 2021 Cumulus Networks, inc.
# All Rights reserved.


###############################
# port-mirror
###############################

def mirror_get(ctx, rev):
    if rev != "operational":
        return ctx.config_v1.getPortMirror(rev)

    return ctx.switchd_v1.getPortMirror()


########################################
# port-mirror/session
########################################

def sessions_get(ctx, rev):
    mirror = mirror_get(ctx, rev)
    return mirror.get('session', {})


########################################
# port-mirror/session/{session_id}
########################################


def session_get(ctx, rev, session_id):
    sessions = sessions_get(ctx, rev)
    return sessions.get(session_id, {})


##########################################
# port-mirror/session/{session_id}/span
##########################################


def span_get(ctx, rev, session_id):
    session = session_get(ctx, rev, session_id)
    return session.get('span', {})


#################################################
# port-mirror/session/{session_id}/span/source-port
#################################################


def span_src_ports_get(ctx, rev, session_id):
    span = span_get(ctx, rev, session_id)
    return span.get('source-port', {})


#################################################
# port-mirror/session/{session_id}/span/source-port/{port-id}
#################################################


def span_src_port_get(ctx, rev, session_id, port_id):
    src_ports = span_src_ports_get(ctx, rev, session_id)
    return src_ports.get(port_id, {})


####################################################
# port-mirror/session/{session_id}/span/destination
####################################################


def span_destinations_get(ctx, rev, session_id):
    span = span_get(ctx, rev, session_id)
    return span.get('destination', {})

####################################################
# port-mirror/session/{session_id}/span/destination/{port-id}
####################################################


def span_destination_get(ctx, rev, session_id, port_id):
    destination = span_destinations_get(ctx, rev, session_id)
    return destination.get(port_id, {})

####################################################
# port-mirror/session/{session_id}/span/truncate
####################################################


def span_truncate_get(ctx, rev, session_id):
    span = span_get(ctx, rev, session_id)
    return span.get('truncate', {})


##########################################
# port-mirror/session/{session_id}/erspan
##########################################

def erspan_get(ctx, rev, session_id):
    session = session_get(ctx, rev, session_id)
    return session.get('erspan', {})


##########################################
# port-mirror/session/{session_id}/erspan/source-port
##########################################

def erspan_src_ports_get(ctx, rev, session_id):
    erspan = erspan_get(ctx, rev, session_id)
    return erspan.get('source-port', {})

##########################################
# port-mirror/session/{session_id}/erspan/source-port/{port-id}
##########################################


def erspan_src_port_get(ctx, rev, session_id, port_id):
    source_ports = erspan_src_ports_get(ctx, rev, session_id)
    return source_ports.get(port_id, {})

##########################################
# port-mirror/session/{session_id}/erspan/destination
##########################################


def erspan_destination_get(ctx, rev, session_id):
    erspan = erspan_get(ctx, rev, session_id)
    return erspan.get('destination', {})

##########################################
# port-mirror/session/{session_id}/erspan/destination/source-ip
##########################################


def erspan_destination_src_ips_get(ctx, rev, session_id):
    destination = erspan_destination_get(ctx, rev, session_id)
    return destination.get('source-ip', {})


def erspan_destination_src_ip_get(ctx, rev, session_id, source_ip):
    source_ips = erspan_destination_src_ips_get(ctx, rev, session_id)
    return source_ips.get(source_ip, {})

##########################################
# port-mirror/session/{session_id}/erspan/destination/dest-ip
##########################################


def erspan_destination_dest_ips_get(ctx, rev, session_id):
    destination = erspan_destination_get(ctx, rev, session_id)
    return destination.get('dest-ip', {})


def erspan_destination_dest_ip_get(ctx, rev, session_id, dest_ip):
    destination_ips = erspan_destination_dest_ips_get(ctx, rev, session_id)
    return destination_ips.get(dest_ip, {})

##########################################
# port-mirror/session/{session_id}/erspan/truncate
##########################################


def erspan_truncate_get(ctx, rev, session_id):
    erspan = erspan_get(ctx, rev, session_id)
    return erspan.get('truncate', {})
