# Copyright (C) 2018-2021 Cumulus Networks, inc
#
# All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
#
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt


###############################
# Router
###############################

def router_get(ctx, rev):
    if rev != "operational":
        return ctx.config_v1.getRouter(rev)

    # We will not show the operational state as of now.
    raise ctx.errors.OperationalNotSupported


###############################
# Router/nexthop-group
###############################

# nhgs_path = ('nexthop-group',)


def nexthop_groups_get(ctx, rev):
    return router_get(ctx, rev)["nexthop-group"]


###############################
# Router/nexthop-group/{nexthop-group-id}
###############################

def nexthop_group_get(ctx, rev, nexthop_group_id):
    if rev != "operational":
        return nexthop_groups_get(ctx, rev).get(nexthop_group_id, {})

    # We will not show the operational state as of now.
    raise ctx.errors.OperationalNotSupported


###############################
# Router/nexthop-group/{nexthop-group-id}/via
###############################


def nhg_vias_get(ctx, rev, nexthop_group_id):
    if rev != "operational":
        return nexthop_group_get(ctx, rev, nexthop_group_id).get(
            'via', {})

    # We will not show the operational state as of now.
    raise ctx.errors.OperationalNotSupported


###############################
# Router/nexthop-group/{nexthop-group-id}/via/{via-id}
###############################


def nhg_via_get(ctx, rev, nexthop_group_id, via_id):
    if rev != "operational":
        return nhg_vias_get(ctx, rev, nexthop_group_id).get(
            via_id, {})

    # We will not show the operational state as of now.
    raise ctx.errors.OperationalNotSupported


###############################
# Router/pbr/map
###############################

pbr_map_path = ("pbr", "map")


def pbr_maps_get(ctx, rev):
    if rev != "operational":
        return ctx.config_v1.getPbrMaps(rev)

    # We will not show the operational state as of now.
    raise ctx.errors.OperationalNotSupported


###############################
# Router/pbr
###############################

def pbr_get(ctx, rev):
    if rev != "operational":
        return ctx.config_v1.getPbr(rev)

    # We will not show the operational state as of now.
    raise ctx.errors.OperationalNotSupported


###############################
# Router/pbr/map/{pbr-map-id}
###############################

def pbr_map_get(ctx, rev, pbr_map_id):
    if rev != "operational":
        return ctx.config_v1.getPbrMap(rev, pbr_map_id)

    # We will not show the operational state as of now.
    raise ctx.errors.OperationalNotSupported


###############################
# Router/pbr/map/{pbr-map-id}/rule
###############################

def pbr_map_rules_get(ctx, rev, pbr_map_id):
    if rev != "operational":
        return ctx.config_v1.getPbrMap(rev, pbr_map_id).get('rule', {})

    # We will not show the operational state as of now.
    raise ctx.errors.OperationalNotSupported


###############################
# Router/pbr/map/{pbr-map-id}/rule/{rule-id}
###############################

def pbr_map_rule_get(ctx, rev, pbr_map_id, rule_id):
    if rev != "operational":
        return pbr_map_rules_get(ctx, rev, pbr_map_id).get(rule_id, {})

    # We will not show the operational state as of now.
    raise ctx.errors.OperationalNotSupported


###############################
# Router/pbr/map/{pbr-map-id}/rule/{rule-id}/match
###############################

def pbr_map_rule_match_get(ctx, rev, pbr_map_id, rule_id):
    if rev != "operational":
        return pbr_map_rule_get(ctx, rev, pbr_map_id, rule_id).get('match', {})

    # We will not show the operational state as of now.
    raise ctx.errors.OperationalNotSupported


###############################
# Router/pbr/map/{pbr-map-id}/rule/{rule-id}/action
###############################

def pbr_map_rule_action_get(ctx, rev, pbr_map_id, rule_id):
    if rev != "operational":
        return pbr_map_rule_get(ctx, rev, pbr_map_id, rule_id).get('action',
                                                                   {})

    # We will not show the operational state as of now.
    raise ctx.errors.OperationalNotSupported


###############################
# Router/policy
###############################

def policy_get(ctx, rev):
    return router_get(ctx, rev)["policy"]


###############################
# Router/policy/community-lists
###############################

def community_lists_get(ctx, rev):
    return policy_get(ctx, rev)["community-list"]


#####################################################
# Router/policy/community-lists/{list-id}
####################################################

def community_list_get(ctx, rev, list_id):
    clists = community_lists_get(ctx, rev)
    try:
        return clists[list_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


####################################################################
# Router/policy/community-lists/{list-id}/rule
###################################################################

def community_list_rules_get(ctx, rev, list_id):
    clist = community_list_get(ctx, rev, list_id)
    return clist["rule"]


####################################################################
# Router/policy/community-lists/{list-id}/rule/{rule-id}
###################################################################

def community_list_rule_get(ctx, rev, list_id, rule_id):
    rules = community_list_rules_get(ctx, rev, list_id)
    try:
        return rules[rule_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


############################################################################
# Router/policy/community-lists/{list-id}/rule/{rule-id}/community
############################################################################

def rule_communities_get(ctx, rev, list_id, rule_id):
    rule = community_list_rule_get(ctx, rev, list_id, rule_id)
    return rule["community"]


############################################################################
# Router/policy/community-lists/{list-id}/rule/{rule-id}/community/{community-id}
############################################################################

def rule_community_get(ctx, rev, list_id, rule_id, community_id):
    communities = rule_communities_get(ctx, rev, list_id, rule_id)
    try:
        return communities[community_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


###############################
# Router/policy/as-path-lists
###############################

def aspath_lists_get(ctx, rev):
    return policy_get(ctx, rev)["as-path-list"]


#####################################################
# Router/policy/as-path-lists/{list-id}
####################################################

def aspath_list_get(ctx, rev, list_id):
    clists = aspath_lists_get(ctx, rev)
    try:
        return clists[list_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


####################################################################
# Router/policy/as-path-lists/{list-id}/rule
###################################################################

def aspath_list_rules_get(ctx, rev, list_id):
    clist = aspath_list_get(ctx, rev, list_id)
    return clist["rule"]


####################################################################
# Router/policy/as-path-lists/{list-id}/rule/{rule-id}
###################################################################

def aspath_list_rule_get(ctx, rev, list_id, rule_id):
    rules = aspath_list_rules_get(ctx, rev, list_id)
    try:
        return rules[rule_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


######################################
# Router/policy/ext-community-lists
######################################

def ext_community_lists_get(ctx, rev):
    return policy_get(ctx, rev)["ext-community-list"]


##############################################################
# Router/policy/ext-community-lists/{list-id}
##############################################################

def ext_community_list_get(ctx, rev, list_id):
    clists = ext_community_lists_get(ctx, rev)
    try:
        return clists[list_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


####################################################################
# Router/policy/ext-community-lists/{list-id}/rule
###################################################################

def ext_community_list_rules_get(ctx, rev, list_id):
    clist = ext_community_list_get(ctx, rev, list_id)
    return clist["rule"]


####################################################################
# Router/policy/community-lists/{community-list-id}/rule/{rule-id}
###################################################################

def ext_community_list_rule_get(ctx, rev, list_id, rule_id):
    rules = ext_community_list_rules_get(ctx, rev, list_id)
    try:
        return rules[rule_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


############################################################################
# Router/policy/ext-community-lists/{list-id}/
# rule/{rule-id}/ext-community
############################################################################

def rule_ext_community_get(ctx, rev, list_id, rule_id):
    rule = ext_community_list_rule_get(ctx, rev, list_id,
                                       rule_id)
    return rule["ext-community"]


############################################################################
# Router/policy/ext-community-lists/{list-id}/
# rule/{rule-id}/ext-community/rt
############################################################################

def rule_ext_community_rts_get(ctx, rev, list_id, rule_id):
    ext_com = rule_ext_community_get(ctx, rev, list_id, rule_id)
    return ext_com["rt"]


############################################################################
# Router/policy/ext-community-lists/{list-id}/
# rule/{rule-id}/ext-community/rt/{ext-community-id}
############################################################################

def rule_ext_community_rt_get(ctx, rev, list_id, rule_id, ext_community_id):
    rts = rule_ext_community_rts_get(ctx, rev, list_id, rule_id)
    try:
        return rts[ext_community_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


############################################################################
# Router/policy/ext-community-lists/{list-id}/
# rule/{rule-id}/ext-community/soo
############################################################################

def rule_ext_community_soos_get(ctx, rev, list_id, rule_id):
    ext_com = rule_ext_community_get(ctx, rev, list_id, rule_id)
    return ext_com["soo"]


############################################################################
# Router/policy/ext-community-lists/{list-id}/
# rule/{rule-id}/ext-community/soo/{ext-community-id}
############################################################################

def rule_ext_community_soo_get(ctx, rev, list_id, rule_id, ext_community_id):
    soos = rule_ext_community_soos_get(ctx, rev, list_id, rule_id)
    try:
        return soos[ext_community_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


############################################
# Router/policy/large-community-lists
###########################################

def large_community_lists_get(ctx, rev):
    return policy_get(ctx, rev)["large-community-list"]


##################################################################
# Router/policy/large-community-lists/{list-id}
#################################################################

def large_community_list_get(ctx, rev, list_id):
    clists = large_community_lists_get(ctx, rev)
    try:
        return clists[list_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


####################################################################
# Router/policy/large-community-lists/{list-id}/rule
###################################################################

def large_community_list_rules_get(ctx, rev, list_id):
    clist = large_community_list_get(ctx, rev, list_id)
    return clist["rule"]


##############################################################################
# Router/policy/large-community-lists/{list-id}/rule/{rule-id}
##############################################################################

def large_community_list_rule_get(ctx, rev, list_id, rule_id):
    rules = large_community_list_rules_get(ctx, rev, list_id)
    try:
        return rules[rule_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


###########################################################################
# Router/policy/large-community-lists/{list-id}/
# rule/{rule-id}/large-community
#########################################################################

def rule_large_communities_get(ctx, rev, list_id, rule_id):
    rule = large_community_list_rule_get(ctx, rev, list_id,
                                         rule_id)
    return rule["large-community"]


###########################################################################
# Router/policy/large-community-lists/{list-id}/
# rule/{rule-id}/large-community/{large-community-id}
#########################################################################

def rule_large_community_get(ctx, rev, list_id, rule_id, large_community_id):
    communities = rule_large_communities_get(ctx, rev, list_id, rule_id)
    try:
        return communities[large_community_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


###############################
# Router/policy/prefix-lists
###############################

def prefix_lists_get(ctx, rev):
    return policy_get(ctx, rev)["prefix-list"]


##############################################
# Router/policy/prefix-list/{prefix-list-id}
#############################################

def prefix_list_get(ctx, rev, prefix_list_id):
    lists = prefix_lists_get(ctx, rev)
    try:
        return lists[prefix_list_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


####################################################
# Router/policy/prefix-list/{prefix-list-id}/rules
####################################################

def prefix_list_rules_get(ctx, rev, prefix_list_id):
    return prefix_list_get(ctx, rev, prefix_list_id)["rule"]


#############################################################
# Router/policy/prefix-list/{prefix-list-id}/rule/{rule-id}
#############################################################

def prefix_list_rule_get(ctx, rev, prefix_list_id, rule_id):
    rules = prefix_list_rules_get(ctx, rev, prefix_list_id)
    try:
        return rules[rule_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


#################################################################
# Router/policy/prefix-list/{prefix-list-id}/rule/{rule-id}/match
#################################################################

def prefix_list_rule_matches_get(ctx, rev, prefix_list_id, rule_id):
    return prefix_list_rule_get(ctx, rev, prefix_list_id, rule_id)["match"]


############################################################################
# Router/policy/prefix-list/{prefix-list-id}/rule/{rule-id}/match/{match-id}
###########################################################################

def prefix_list_rule_match_get(ctx, rev, prefix_list_id, rule_id, match_id):
    matches = prefix_list_rule_matches_get(ctx, rev, prefix_list_id, rule_id)
    try:
        return matches[match_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


###############################
# Router/policy/route-maps
###############################

def routemaps_get(ctx, rev):
    return policy_get(ctx, rev)["route-map"]


#########################################
# Router/policy/route-map/{route-map-id}
#########################################

def routemap_get(ctx, rev, route_map_id):
    maps = routemaps_get(ctx, rev)
    try:
        return maps[route_map_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


###############################################
# Router/policy/route-map/{route-map-id}/rule
###############################################

def routemap_rules_get(ctx, rev, route_map_id):
    return routemap_get(ctx, rev, route_map_id)["rule"]


#######################################################
# Router/policy/route-map/{route-map-id}/rule/rule-id
######################################################

def routemap_rule_get(ctx, rev, route_map_id, rule_id):
    rules = routemap_rules_get(ctx, rev, route_map_id)
    try:
        return rules[rule_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


############################################################
# Router/policy/route-map/{route-map-id}/rule/rule-id/match
############################################################

def routemap_rule_match_get(ctx, rev, route_map_id, rule_id):
    return routemap_rule_get(ctx, rev, route_map_id, rule_id)["match"]


############################################################
# Router/policy/route-map/{route-map-id}/rule/rule-id/set
############################################################

def routemap_rule_set_get(ctx, rev, route_map_id, rule_id):
    return routemap_rule_get(ctx, rev, route_map_id, rule_id)["set"]


###########################################################################
# Router/policy/route-map/{route-map-id}/rule/{rule-id}/set/as-path-prepend
###########################################################################

def routemap_as_path_prepend_get(ctx, rev, route_map_id, rule_id):
    rule_set = routemap_rule_set_get(ctx, rev, route_map_id, rule_id)
    return rule_set["as-path-prepend"]


####################################################################
# Router/policy/route-map/{route-map-id}/rule/{rule-id}/set/community
####################################################################

def routemap_communities_get(ctx, rev, route_map_id, rule_id):
    rule_set = routemap_rule_set_get(ctx, rev, route_map_id, rule_id)
    return rule_set["community"]


####################################################################
# Router/policy/route-map/{route-map-id}/rule/{rule-id}/set/
# community/{community-id}
####################################################################

def routemap_community_get(ctx, rev, route_map_id, rule_id, community_id):
    comms = routemap_communities_get(ctx, rev, route_map_id, rule_id)
    try:
        return comms[community_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


############################################################################
# Router/policy/route-map/{route-map-id}/rule/{rule-id}/set/large-community
############################################################################

def routemap_large_communities_get(ctx, rev, route_map_id, rule_id):
    rule_set = routemap_rule_set_get(ctx, rev, route_map_id, rule_id)
    return rule_set["large-community"]


#########################################################################
# Router/policy/route-map/{route-map-id}/rule/{rule-id}/set/
# large-community/{large-community-id}
#########################################################################

def routemap_large_community_get(ctx, rev, route_map_id, rule_id,
                                 large_community_id):
    comms = routemap_large_communities_get(ctx, rev, route_map_id, rule_id)
    try:
        return comms[large_community_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


############################################################################
# Router/policy/route-map/{route-map-id}/rule/{rule-id}/set/aggregator-as
############################################################################

def routemap_aggregator_ases_get(ctx, rev, route_map_id, rule_id):
    rule_set = routemap_rule_set_get(ctx, rev, route_map_id, rule_id)
    return rule_set["aggregator-as"]


#########################################################################
# Router/policy/route-map/{route-map-id}/rule/{rule-id}/set/
# aggregator-as/{asn-id}
#########################################################################

def routemap_aggregator_as_get(ctx, rev, route_map_id, rule_id, asn_id):
    aggs = routemap_aggregator_ases_get(ctx, rev, route_map_id, rule_id)
    try:
        return aggs[asn_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


#########################################################################
# Router/policy/route-map/{route-map-id}/rule/{rule-id}/set/
# aggregator-as/{asn-id}/address
#########################################################################

def routemap_aggregator_as_addrs_get(ctx, rev, route_map_id, rule_id, asn_id):
    aggs = routemap_aggregator_as_get(ctx, rev, route_map_id, rule_id, asn_id)
    return aggs["address"]


#########################################################################
# Router/policy/route-map/{route-map-id}/rule/{rule-id}/set/
# aggregator-as/{asn-id}/address/{ipv4-address-id}
#########################################################################

def routemap_aggregator_as_addr_get(ctx, rev, route_map_id, rule_id,
                                    asn_id, ipv4_address_id):
    aggs = routemap_aggregator_as_addrs_get(ctx, rev, route_map_id,
                                            rule_id, asn_id)
    try:
        return aggs[ipv4_address_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


############################################################
# Router/policy/route-map/{route-map-id}/rule/rule-id/action
############################################################

def routemap_rule_action_get(ctx, rev, route_map_id, rule_id):
    return routemap_rule_get(ctx, rev, route_map_id, rule_id)["action"]


####################################################################
# Router/policy/route-map/{route-map-id}/rule/rule-id/action/permit
####################################################################

def action_deny_get(ctx, rev, route_map_id, rule_id):
    return routemap_rule_action_get(ctx, rev, route_map_id, rule_id)["deny"]


####################################################################
# Router/policy/route-map/{route-map-id}/rule/rule-id/action/permit
####################################################################

def action_permit_get(ctx, rev, route_map_id, rule_id):
    return routemap_rule_action_get(ctx, rev, route_map_id, rule_id)["permit"]


###############################################################################
# Router/policy/route-map/{route-map-id}/rule/rule-id/action/permit/exit-policy
###############################################################################

def action_permit_exit_policy_get(ctx, rev, route_map_id, rule_id):
    return action_permit_get(ctx, rev, route_map_id, rule_id)["exit-policy"]


###############################
# BGP
###############################

def bgp_get(ctx, rev):
    if rev != "operational":
        return ctx.config_v1.getBgp(rev)

    # We will not show the operational state as of now.
    raise ctx.errors.OperationalNotSupported


###############################
# Router/BGP/graceful-restart
###############################

def bgp_graceful_restart_get(ctx, rev):
    return bgp_get(ctx, rev)["graceful-restart"]


###############################
# Router/BGP/graceful-restart
###############################

# TODO: CUE-3808
def bgp_convergence_wait_get(ctx, rev):
    return bgp_get(ctx, rev)["convergence-wait"]


###############################
# OSPF
###############################

def ospf_get(ctx, rev):
    if rev != "operational":
        return ctx.config_v1.getOspf(rev)

    # We will not show the operational state as of now.
    raise ctx.errors.OperationalNotSupported


###############################
# OSPF/timers
###############################

def ospf_timers_get(ctx, rev):
    return ospf_get(ctx, rev)["timers"]


###############################
# OSPF/lsa-timers
###############################

def ospf_lsa_timers_get(ctx, rev):
    return ospf_timers_get(ctx, rev)["lsa"]


###############################
# OSPF/spf-timers
###############################

def ospf_spf_timers_get(ctx, rev):
    return ospf_timers_get(ctx, rev)["spf"]


###############################
# PIM
###############################

def pim_get(ctx, rev):
    if rev != "operational":
        return ctx.config_v1.getPim(rev)

    # We will not show the operational state as of now.
    raise ctx.errors.OperationalNotSupported


###############################
# PIM/timers
###############################

def pim_timers_get(ctx, rev):
    return pim_get(ctx, rev)["timers"]


###############################
# Router/igmp
###############################

def igmp_get(ctx, rev):
    if rev != "operational":
        return ctx.config_v1.getRouter(rev).get('igmp', {})

    # We will not show the operational state as of now.
    raise ctx.errors.OperationalNotSupported


###############################
# Router/vrrp
###############################

def vrrp_get(ctx, rev):
    if rev != "operational":
        return ctx.config_v1.getRouter(rev).get('vrrp', {})

    # We will not show the operational state as of now.
    raise ctx.errors.OperationalNotSupported


###############################
# Router/vrr
###############################

def vrr_get(ctx, rev):
    if rev != "operational":
        return ctx.config_v1.getRouter(rev).get('vrr', {})

    # We will not show the operational state as of now.
    raise ctx.errors.OperationalNotSupported
