#
# Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software product is a proprietary product of Nvidia Corporation
# and its affiliates (the "Company") and all right, title, and interest
# in and to the software product, including all associated intellectual
# property rights, are and shall remain exclusively with the Company.
#
# This software product is governed by the End User License Agreement
# provided with the software product.
# Copyright (C) 2021 Cumulus Networks, inc.
# All Rights reserved.
# Cumulus Linux services: /usr/share/cumulus/EULA.txt


###############################
# Service
###############################

def service_get(ctx, rev):
    if rev != "operational":
        return ctx.config_v1.getService(rev)

    # Todo
    return {}


###############################
# syslog
###############################

def syslogs_get(ctx, rev):
    if rev != "operational":
        return ctx.config_v1.getSyslogs(rev)

    # Todo
    return {}


###############################
# syslog/{vrf-id}
###############################

def syslog_get(ctx, rev, vrf_id):
    if rev != "operational":
        return ctx.config_v1.getSyslog(rev, vrf_id)

    # Todo
    return {}


###############################
# syslog/{vrf-id}/servers
###############################

def syslog_servers_get(ctx, rev, vrf_id):
    if rev != "operational":
        slog = syslog_get(ctx, rev, vrf_id)
        return slog["server"]
    return ctx.rsyslog_v1.getRsyslogServers(vrf_id)


###############################
# syslog/{vrf-id}/server
###############################

def syslog_server_get(ctx, rev, vrf_id, server_id):
    servers = syslog_servers_get(ctx, rev, vrf_id)
    try:
        return servers[server_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


###############################
# LLDP
###############################

def lldp_get(ctx, rev):
    if rev != "operational":
        return ctx.config_v1.getLldp(rev)
    return ctx.lldp_v1.getLldpConfig()


###############################
# DNS
###############################

def dnss_get(ctx, rev):
    if rev != "operational":
        return ctx.config_v1.getDnss(rev)

    # Todo
    return {}


###############################
# DNS/{vrf_id}
###############################

def dns_get(ctx, rev, vrf_id):
    if rev != "operational":
        return ctx.config_v1.getDns(rev, vrf_id)

    # Todo
    return {}


###############################
# DNS/{vrf_id}/server
###############################

def dns_servers_get(ctx, rev, vrf_id):
    if rev != "operational":
        return ctx.config_v1.getDns(rev, vrf_id)["server"]

    try:
        return ctx.dns_v1.getServers()[vrf_id]["server"]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist


def dns_server_get(ctx, rev, vrf_id, dns_server_id):
    if rev != "operational":
        dns = ctx.config_v1.getDns(rev, vrf_id)
        return dns["server"][dns_server_id]


# def host_entries_get(ctx, rev):
#    if rev != "operational":
#        return ctx.config_v1.getHostEntries(rev)
#
#    return ctx.dns_v1.getHostEntries()
#
#
# def host_entry_get(ctx, rev, host_entry_id):
#    if rev != "operational":
#        return ctx.config_v1.getHostEntry(rev, host_entry_id)
#
#    return ctx.dns_v1.getHostEntry(host_entry_id)
