# Copyright (C) 2018-2021 Cumulus Networks, inc
#
# All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
#
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

from cue.exceptions import NotFound


###############################
# VxLAN
###############################
def vxlan_get(ctx, rev):
    if rev == 'operational':
        # VxLAN config does not have a operational revision
        rev = 'applied'
    return ctx.config_v1.getNveVxlan(rev)


###############################
# VxLAN Source
###############################

def vxlan_source_get(ctx, rev):
    return vxlan_get(ctx, rev)["source"]


###############################
# VxLAN MLAG
###############################

def vxlan_mlag_get(ctx, rev):
    return vxlan_get(ctx, rev)["mlag"]


###############################
# VxLAN Flooding
###############################

def vxlan_flooding_get(ctx, rev):
    output = vxlan_get(ctx, rev)
    try:
        return output["flooding"]
    except KeyError:
        raise NotFound


###############################
# VxLAN Flooding head-end-replications
###############################

def vxlan_hreps_get(ctx, rev):
    output = vxlan_flooding_get(ctx, rev)
    try:
        return output["head-end-replication"]
    except KeyError:
        raise NotFound


###############################
# VxLAN Flooding head-end-replication
###############################

def vxlan_hrep_get(ctx, rev, hrep_id):

    hreps = vxlan_hreps_get(ctx, rev)
    try:
        return hreps[hrep_id]
    except KeyError:
        raise ctx.errors.ItemDoesNotExist
