#
# Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software product is a proprietary product of Nvidia Corporation
# and its affiliates (the "Company") and all right, title, and interest
# in and to the software product, including all associated intellectual
# property rights, are and shall remain exclusively with the Company.
#
# This software product is governed by the End User License Agreement
# provided with the software product.
# Copyright (C) 2021 Cumulus Networks, inc.
# All Rights reserved.


import json
import logging
from pydash import py_


logger = logging.getLogger(__name__)
ptp_domain = 0
ptp_domain_read = 0

###############################
# Raw
###############################


def ptp4l_cli_raw_get(ctx, cmd_id, arg_id=None):
    global ptp_domain_read
    global ptp_domain
    rd_lst = 0
    fd = {}

    if not ptp_domain_read:
        ptp_conf = ctx.linuxptp_v1.getCuev1Ptp4lConfPtp()
        if 'domain' in ptp_conf:
            ptp_domain = ptp_conf['domain']
        else:
            ptp_domain = 0
        ptp_domain_read = 1
    try:
        s = ''
        if arg_id is None:
            if (cmd_id == 'ts-log'):
                s = ctx.sh.sudo("/usr/lib/cumulus/tsmon", "-x", "-j")
                rd_lst = 1
            elif (cmd_id == 'max-offset-log'):
                s = ctx.sh.sudo("/usr/lib/cumulus/tsmon", "-m", "-j")
                rd_lst = 1
            elif (cmd_id == 'min-offset-log'):
                s = ctx.sh.sudo("/usr/lib/cumulus/tsmon", "-n", "-j")
                rd_lst = 1
            elif (cmd_id == 'mpd-log'):
                s = ctx.sh.sudo("/usr/lib/cumulus/tsmon", "-p", "-j")
                rd_lst = 1
            elif (cmd_id == 'violations'):
                s = ctx.sh.sudo("/usr/lib/cumulus/tsmon", "-v", "-j")
            else:
                return str(ctx.sh.sudo("/usr/lib/nvue/ptp4l-cli", cmd_id,
                           ptp_domain))
            if rd_lst == 1:
                lst = s.split('\n')
                for st in lst:
                    ts = {}
                    if len(st) == 0:
                        break
                    a, b = st.split('@')
                    for element in b.split(','):
                        c, d = element.split('=')
                        if c.strip() != 'log-time':
                            ts[c.strip()] = int(d.strip())
                        else:
                            ts[c.strip()] = d.strip()
                    fd[a.strip()] = ts
            else:
                for element in s.split(','):
                    c, d = element.split('=')
                    if c.strip() == 'min-offset-count':
                        fd[c.strip()] = int(d.strip())
                    elif c.strip() == 'max-offset-count':
                        fd[c.strip()] = int(d.strip())
                    elif c.strip() == 'path-delay-count':
                        fd[c.strip()] = int(d.strip())
                    else:
                        fd[c.strip()] = d.strip()
            return json.dumps(fd)
        else:
            return str(ctx.sh.sudo("/usr/lib/nvue/ptp4l-cli", cmd_id, arg_id,
                       ptp_domain))
    except ctx.sh.ErrorReturnCode_1 as ex:
        return ex.stdout.decode()


###############################
# Native
###############################

def ptp4l_cli_get(ctx, cmd_id, arg_id=None):
    try:
        return json.loads(ctx.linuxptp_v1.getRawPtp4lCli(cmd_id, arg_id))
    except json.JSONDecodeError:
        logger.info("Failed to parse ouput from ptp4l cli")
        return {}


def current_ds_get(ctx, instance_id):
    return ctx.linuxptp_v1.getPtp4lCli("current")


def parent_ds_get(ctx, instance_id):
    return ctx.linuxptp_v1.getPtp4lCli("parent")


def timeprops_ds_get(ctx, instance_id):
    return ctx.linuxptp_v1.getPtp4lCli("props")


def master_ds_get(ctx, instance_id):
    return ctx.linuxptp_v1.getPtp4lCli("master")


def default_ds_get(ctx, instance_id):
    return ctx.linuxptp_v1.getPtp4lCli("default")


def port_ds_get(ctx, instance_id, interface_id):
    return ctx.linuxptp_v1.getPtp4lCli("port", interface_id)


def stats_ds_get(ctx, instance_id, interface_id):
    return ctx.linuxptp_v1.getPtp4lCli("stats", interface_id)


def ts_log_get(ctx, instance_id):
    return ctx.linuxptp_v1.getPtp4lCli("ts-log")


def violations_get(ctx, instance_id):
    return ctx.linuxptp_v1.getPtp4lCli("violations")


def am_violations_get(ctx, instance_id):
    return ctx.linuxptp_v1.getPtp4lCli("log", "am")


def fm_violations_get(ctx, instance_id):
    return ctx.linuxptp_v1.getPtp4lCli("log", "fm")


def max_offset_log_get(ctx, instance_id):
    return ctx.linuxptp_v1.getPtp4lCli("max-offset-log")


def min_offset_log_get(ctx, instance_id):
    return ctx.linuxptp_v1.getPtp4lCli("min-offset-log")


def mpd_log_get(ctx, instance_id):
    return ctx.linuxptp_v1.getPtp4lCli("mpd-log")


###############################
# Translated
###############################

def cuev1_current_ptp_current_get(ctx):
    return ctx.linuxptp_v1.getPtp4lCurrentDS(1)


def cuev1_parent_ptp_parent_get(ctx):
    return ctx.linuxptp_v1.getPtp4lParentDS(1)


def cuev1_time_ptp_timeprops_get(ctx):
    return ctx.linuxptp_v1.getPtp4lTimePropsDS(1)


def cuev1_default_ptp_clock_quality_get(ctx):
    default_ds = ctx.linuxptp_v1.getPtp4lDefaultDS(1)
    cq = default_ds["clock-quality"]

    # Patch up a mispelling in the tool
    if 'offset-sclaled-log-variance' in cq:
        cq = dict(cq)
        cq['offset-scaled-log-variance'] = cq['offset-sclaled-log-variance']
        del cq['offset-sclaled-log-variance']

    return cq


def cuev1_port_if_ptp_get(ctx, interface_id):
    port_ds = ctx.linuxptp_v1.getPtp4lPortDS(1, interface_id)
    return {
        'port-state': port_ds['port-state'],
        'peer-mean-path-delay': port_ds['peer-mean-path-delay'],
        'protocol-version': port_ds['protocol-version']
    }


def cuev1_stats_if_ptp_get(ctx, interface_id):
    stats_ds = ctx.linuxptp_v1.getPtp4lStatsDS(1, interface_id)
    # The native layer closely matches the cue_v1 model.  We just
    # need to remove a few fields.
    return py_.omit(stats_ds, "port_id")


def cuev1_ptp_timestamp_logs_get(ctx):
    return ctx.linuxptp_v1.getPtp4lTsLogs(1)


def cuev1_ptp_violations_get(ctx):
    return ctx.linuxptp_v1.getPtp4lViolations(1)


def cuev1_ptp_am_violations_log_get(ctx):
    return ctx.linuxptp_v1.getPtp4lAmVioLog(1)


def cuev1_ptp_fm_violations_log_get(ctx):
    return ctx.linuxptp_v1.getPtp4lFmVioLog(1)


def cuev1_ptp_max_offset_violations_log_get(ctx):
    return ctx.linuxptp_v1.getPtp4lMaxOffsetVioLog(1)


def cuev1_ptp_min_offset_violations_log_get(ctx):
    return ctx.linuxptp_v1.getPtp4lMinOffsetVioLog(1)


def cuev1_ptp_mpd_violations_log_get(ctx):
    return ctx.linuxptp_v1.getPtp4lMpdVioLog(1)
