#
# Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software product is a proprietary product of Nvidia Corporation
# and its affiliates (the "Company") and all right, title, and interest
# in and to the software product, including all associated intellectual
# property rights, are and shall remain exclusively with the Company.
#
# This software product is governed by the End User License Agreement
# provided with the software product.
from shlex import split
from cue_versions_v1.revman import UnknownRevisionError
from cue.exceptions import NotFound, BadRequest


AVAILABLE_SUBCOMMANDS = {"show", "log"}


def git_raw_get(ctx, revision_id, cmd):
    cmd_tokens = split(cmd)
    if not (cmd_tokens and cmd_tokens[0] in AVAILABLE_SUBCOMMANDS):
        raise BadRequest(f"{cmd!r} not available.")
    try:
        with ctx.versions_v1._repo.get_revman(revision_id) as revman:
            output = ctx.sh.contrib.git(
                *cmd_tokens,
                _cwd=revman.root
            )
    except UnknownRevisionError:
        raise NotFound()
    return str(output)
