#
# Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software product is a proprietary product of Nvidia Corporation
# and its affiliates (the "Company") and all right, title, and interest
# in and to the software product, including all associated intellectual
# property rights, are and shall remain exclusively with the Company.
#
# This software product is governed by the End User License Agreement
# provided with the software product.


def _should_include_in_history(commit):
    is_merge = len(commit['parent-hashes']) > 1
    has_meta = 'parsed-body' in commit
    return has_meta and is_merge


def _translate_commit_for_history(commit):
    item = {
        'message': commit['subject'],
        'date': commit['date'],
        'ref': next((
            refname
            for refname in commit['ref-names']
            if refname.endswith("/done")
        ), commit['hash']),
        'apply-meta': commit['parsed-body'],
    }
    item['apply-id'] = next((
        refname
        for refname in commit['ref-names']
        if refname.startswith('n/')
    ), item['ref'])
    return item


def history_get(ctx, revision_id):
    commits = ctx.versions_v1.getGitLog(revision_id)
    return [
        _translate_commit_for_history(commit)
        for commit in commits
        if _should_include_in_history(commit)
    ]
